package de.berndbock.bin2hex;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * The converter reads binary files and converts the bytes into
 * comma separated hexadecimal values.
 *
 * @author Bernd Bock
 */
public class Bin2HexConverter {

	private static final int bytesPerLine = 16;
	private static final byte CR = 13;
	private static final byte LF = 10;
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
        if (args.length < 2) {
            System.out.println("Please provide input and output files");
            System.exit(0);
        }
 
        String inputFile = args[0];
        String outputFile = args[1];
 
		System.out.println("Bin2HexConverter");
		System.out.println("  input:  " + inputFile);
		System.out.println("  output: " + outputFile);

 
        try (
            InputStream  inputStream  = new FileInputStream(inputFile);
            OutputStream outputStream = new FileOutputStream(outputFile);
        ) {
 
            int byteRead;
            String hex;
            int count = 0;
            boolean first = true;
            int writeCount = 0;
 
            while ((byteRead = inputStream.read()) != -1) {
            	if (first)
            		first = false;
            	else
            		outputStream.write(',');
            	
            	if (count++ >= bytesPerLine) {
            		count = 1;
            		outputStream.write(CR);
            		outputStream.write(LF);
            	}
            	
            	hex = String.format("$%02X", byteRead);
                outputStream.write(hex.getBytes());
        		writeCount++;
            }
            
            System.out.println("bytes written: " + writeCount);
 
        }
        catch (IOException ex) {
            ex.printStackTrace();
        }
    }

}
